package Sub::Sort;

#use 5.010001;
use parent qw(Sort::Sub);

1;
# ABSTRACT: Dummy module to catch Sort::Sub typo

__END__

=pod

=encoding UTF-8

=head1 NAME

Sub::Sort - Dummy module to catch Sort::Sub typo

=head1 VERSION

This document describes version 0.120 of Sub::Sort (from Perl distribution Sort-Sub), released on 2020-05-25.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Sort-Sub>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Sort-Sub>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Sort-Sub>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020, 2019, 2018, 2016, 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
